/*
 * Renderer 1. The MIT License.
 * Copyright (c) 2022 rlkraft@pnw.edu
 * See LICENSE for details.
*/

import renderer.scene.*;
import renderer.scene.primitives.*;
import renderer.scene.util.DrawSceneGraph;
import renderer.framebuffer.*;
import renderer.pipeline.*;

import java.awt.Color;

/**
   Compile and run this program. Look at its output, both
   in the console window and in the PPM file that it creates.
   <p>
   The output in the console window shows you the debugging
   information from the renderer. It shows the results from
   each stage of the rendering pipeline. This program draws
   a single vertex, so the scene's model contains just one
   vertex and one Point primitive.
   <p>
   Look at the output in the console window to see how the
   vertex gets projected into the view plane and then how
   the point gets rasterized into the framebuffer.
   <p>
   Use IrfanView to compare the pixel coordinates listed
   in the console window with the actual pixel in the PPM
   file.
*/
public class One_Vertex_R1
{
   public static void main(String[] args)
   {
      final Model model = new Model("One_Vertex_R1");
      model.addVertex(new Vertex(0.3, 0.6, 0.0));
      model.addPrimitive(new Point(0));

      final Scene scene = new Scene("One_Vertex_R1");
      scene.addPosition(new Position(model, "p0", new Vector(1, 1, -2)));

      // Draw a picture of the scene's tree (DAG) data structure.
      DrawSceneGraph.drawVertexList = true;
      DrawSceneGraph.draw(scene, "One_Vertex_R1_SG");

      final int width  = 200;
      final int height = 200;
      final FrameBuffer fb = new FrameBuffer(width, height, Color.darkGray);

      final FrameBuffer.Viewport vp = fb.new Viewport(50, 50, 100, 100);
      vp.clearVP(Color.black);

      scene.debug = true;
      Rasterize.debug = true;
      Pipeline.render(scene, vp);
      fb.dumpFB2File("One_Vertex_R1.ppm");
      System.err.println("Saved " + "One_Vertex_R1.ppm");
   }
}
